/*
 *  ESP8266 Weather Station / NTP Digital Clock Program
 *
 *  This hardware/software combination implements a weather station
 *  and digital clock. Weather data is streamed over the Internet from
 *  myweather2.com every 15 minutes. The digital clock never needs
 *  setting as it gets the current time and date by polling Network
 *  Time Protocol (NTP) servers on the Internet. This clock's time
 *  is synchronized to NTP time every 5 minutes.
 *
 *  For information about the weather provider see:
 *  http://www.myweather2.com/developer/
 *
 *  The hardware consists of the following parts:
 *    NodeMCU Amica R2 from Electrodragon
 *    1.8" SPI LCD Color Display from Adafruit - Product ID: 358
 *    Pushbutton switch
 *
 *  The connections between the NodeMCU board the the display are
 *  as follows:
 *
 *  ESP8266 Pin  LCD Display Pin       Function
 *  --------------------------------------------------------------------
 *      D3            Lite       Backlight light LED
 *                    MISO       Master In Slave Out (unused)
 *      D5            CLK        SPI Clock
 *      D7            MOSI       Master Out Slave In (data from Esp8266->display)
 *      D8            TFT_CS     Display chip select
 *                    SD_CS      SD card chip select (unused)
 *      D4            D/C        Data/Command toggle
 *                    Reset      Display controller reset (unused)
 *      3V3           Vcc        Power for the display
 *      GND           Gnd        Ground for the display
 *
 *  The Daylight Savings Time (DST) pushbutton switch is connected between
 *  GND and D1 (GPIO5). Every press of this push button toggles clock
 *  between DST and non DST modes. DST status is saved in EEPROM so
 *  it survives power outages.
 *
 *  Concept, Design and Implementation by: Craig A. Lindley
 *  Last Update: 10/30/2015
 */

#include <Adafruit_GFX.h>
#include <ArduinoJson.h>
#include <EEPROM.h>
#include <SPI.h>
#include <Time.h>
#include <ESP8266WiFi.h>

#include "ESP8266_ST7735.h"
#include "TGFunctions.h"
#include "DisplayPages.h"
#include "Misc.h"
#include "NTP.h"
#include "Weather.h"

// ***************************************************************
// Start of user configuration items
// ***************************************************************

// Set your WiFi login credentials
const char * WIFI_SSID = "CraigNet";
const char * WIFI_PASS = "craigandheather";

const int    TIMEZONE_OFFSET = -7;      // Set your timezone offset (-7 is mountain time)
const bool   USE_DST = true;            // Set to false to disable DST mode
const bool   HOUR_FORMAT_12 = true;     // Set to false for 24 hour time mode

const char * API_KEY = "yD9.fEaXf/";    // Key from myweather2.com
const char * LOCATION_STRING = "80908"; // Location indicator

// ***************************************************************
// End of user configuration items
// ***************************************************************

// Connections between NodeMCU module and Adafruit 1.8" LCD display
#define TFT_CS  15
#define TFT_DC   2
#define TFT_LITE 0

// Create LCD driver instance
ESP8266_ST7735 lcd = ESP8266_ST7735(TFT_CS, TFT_DC);

// Create Weather data source instance
Weather weather = Weather(API_KEY, LOCATION_STRING);

// Connection for DST pushbutton
#define DST_SW 5

// Page Display Variables and Data
#define DATA_ACQUISITION_MINS 15
#define DATA_ACQUISITION_MS   (DATA_ACQUISITION_MINS * 60 * 1000)
#define DISPLAY_ADVANCE_SECS  12
#define DISPLAY_ADVANCE_MS    (DISPLAY_ADVANCE_SECS * 1000)

unsigned long dataAcquisitionCount, displayAdvanceCount;

// Page Display Finite State Machine States
enum STATES {INIT, CHECK_EVENTS, ACQUIRE_DATA, ADVANCE_DISPLAY, UPDATE_DST_STATUS};

// Current State
enum STATES state;

// Number of display pages
#define NUMBER_OF_DISPLAY_PAGES 7
int displayPageNumber;

/*******************************************************************/
/***                     Main Program Setup                     ***/
/*******************************************************************/

void setup() {

  // Backlight off until everything is initialized
  pinMode(TFT_LITE, OUTPUT);
  digitalWrite(TFT_LITE, LOW);

  // Initialize serial interface and allow it to stablize
  Serial.begin(115200);
  delay(1000);

  // If using DST, enable the DST push button
  if (USE_DST) {
    // Setup DST pushbutton switch
    pinMode(DST_SW, INPUT_PULLUP);
  }

  // Initialize the LCD (blacktab) display
  lcd.init();

  // Set display orientation to landscape
  lcd.setRotation(3);

  // Clear the display
  clearDisplay();

  Serial.print("\n\nConnecting to: ");
  Serial.println(WIFI_SSID);

  // Attempt WiFi network connection
  WiFi.begin(WIFI_SSID, WIFI_PASS);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nWiFi connected");
  //  Serial.print("IP address: ");
  //  Serial.println(WiFi.localIP());

  // Initialize the NTP time code, set the base timezone and
  // install the time provider
  initNTP(TIMEZONE_OFFSET);

  // Retrieve the weather data
  while (! weather.retrieveWeatherData()) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nGot weather data");

  // Set initial state of page display FSM
  state = INIT;

  // Backlight on
  digitalWrite(TFT_LITE, HIGH);
}

/*******************************************************************/
/***                      Main Program Loop                      ***/
/*******************************************************************/

void loop() {
  // Small delay to keep networking running
  delay(1);

  switch (state) {
    case INIT:

      // Page to be displayed
      displayPageNumber = 1;

      // Show the Wifi/credits page initially
      drawDisplayPageOne();

      // Initialize counters to times in the future
      dataAcquisitionCount = millis() + DATA_ACQUISITION_MS;
      displayAdvanceCount = millis()  + DISPLAY_ADVANCE_MS;

      // Advance to next state
      state = CHECK_EVENTS;
      break;

    case CHECK_EVENTS:

      // Has the DST switch been pressed ?
      if (digitalRead(DST_SW) == LOW) {
        // Advance to next state
        state = UPDATE_DST_STATUS;
        break;
      }

      // Time to acquire new weather data ?
      if (dataAcquisitionCount < millis()) {
        // Advance to next state
        state = ACQUIRE_DATA;
        break;
      }

      // Time to advance display to next page ?
      if (displayAdvanceCount < millis()) {
        // Advance to next state
        state = ADVANCE_DISPLAY;
      }
      break;

    case ACQUIRE_DATA:

      // Retrieve the weather data
      while (! weather.retrieveWeatherData()) {
        // Loop if problems occur
        delay(500);
        Serial.print(".");
      }
      Serial.println("\nGot weather data");

      // Reinitialize counter
      dataAcquisitionCount = millis() + DATA_ACQUISITION_MS;

      // Advance to next state
      state = CHECK_EVENTS;
      break;

    case ADVANCE_DISPLAY:

      // Clear the display
      clearDisplay();

      // Draw the frame around the display
      drawDisplayFrame();

      // Advance the page number to display
      displayPageNumber++;
      if (displayPageNumber > NUMBER_OF_DISPLAY_PAGES) {
        displayPageNumber = 1;
      }

      // Which page is next ?
      switch (displayPageNumber) {
        case 1:
          drawDisplayPageOne();   // WiFi / Credits page
          break;

        case 2:
          drawDisplayPageTwo();   // Current conditions page
          break;

        case 3:
          drawDisplayPageThree(); // Day forecast page
          break;

        case 4:
          drawDisplayPageFour();  // Night forecast page
          break;

        case 5:
          drawDisplayPageFive();  // Next day forecast page
          break;

        case 6:
          drawDisplayPageSix();  // Next night forecast page
          break;

        case 7:
          drawDisplayPageSeven(); // Clock page
          break;
      }

      // Reinitialize counter
      displayAdvanceCount = millis() + DISPLAY_ADVANCE_MS;

      // Advance to next state
      state = CHECK_EVENTS;
      break;

    case UPDATE_DST_STATUS:

      // Toggle the DST enabled bit and inform NTP code for next time sync
      toggleDSTEnabled();

      // Set display page number so page 7 (clock page) will be displayed next
      displayPageNumber = 6;

      // Wait for DST pushbutton to be released
      while (digitalRead(DST_SW) == LOW) {}

      // Advance to next state
      state = ADVANCE_DISPLAY;
      break;
  }
}
