/*
 *  NTP.h - Network Time Protocol Functions
 *
 *  Portions of this code were extracted from the
 *  Time library examples by Michael Margolis and
 *  Paul Stoffregen. Other portions from the NTPClient
 *  example program from the Arduino ESP8266 package.
 *
 *  Concept, Design and Implementation by: Craig A. Lindley
 *  Last Update: 10/30/2015
 */

#ifndef NTP_H
#define NTP_H

extern const bool USE_DST;
extern const bool HOUR_FORMAT_12;

#include <Arduino.h>
#include <ESP8266WiFi.h>
#include <Time.h>
#include <WiFiUdp.h>
#include "Misc.h"

#define LOCALPORT     2390 // Local port to listen for UDP packets
#define NTP_PACKET_SIZE 48 // NTP time stamp is in the first 48 bytes of the message

bool dstEnabled;
int baseTimeZoneOffset;
int realTimeZoneOffset;

byte packetBuffer[NTP_PACKET_SIZE]; // Buffer to hold incoming and outgoing packets

// Don't hardwire the IP address or we won't get the benefits of the time server pool.
IPAddress timeServerIP;
const char *ntpServerName = "time.nist.gov";

// Send an NTP request to the time server at the given address
unsigned long sendNTPpacket(IPAddress& address, WiFiUDP& udp) {

  // Set all bytes in the buffer to 0
  memset(packetBuffer, 0, NTP_PACKET_SIZE);

  // Initialize values needed to form NTP request
  packetBuffer[0] = 0b11100011;   // LI, Version, Mode
  packetBuffer[1] = 0;     // Stratum, or type of clock
  packetBuffer[2] = 6;     // Polling Interval
  packetBuffer[3] = 0xEC;  // Peer Clock Precision
  // 8 bytes of zero for Root Delay & Root Dispersion
  packetBuffer[12]  = 49;
  packetBuffer[13]  = 0x4E;
  packetBuffer[14]  = 49;
  packetBuffer[15]  = 52;

  // All NTP fields have been given values, now
  // you can send a packet requesting a timestamp:
  udp.beginPacket(address, 123); // NTP requests are to port 123
  udp.write(packetBuffer, NTP_PACKET_SIZE);
  udp.endPacket();
}

// NTP Time Provider Code
time_t getNTPTime() {

  // Create UDP instance
  WiFiUDP udp;

  // Set UDP local port
  udp.begin(LOCALPORT);

  int attempts = 10;

  // Try multiple attempts to return the NTP time
  while (attempts--) {

    while (udp.parsePacket() > 0); // Discard any previously received packets

    // Get a server from the pool
    WiFi.hostByName(ntpServerName, timeServerIP);
    //    Serial.print("\nTime server IP address: ");
    //    Serial.println(timeServerIP);

    //    Serial.println("Transmitted NTP Request");
    sendNTPpacket(timeServerIP, udp);

    unsigned long beginWait = millis();
    while (millis() - beginWait < 1500) {
      int size = udp.parsePacket();
      if (size >= NTP_PACKET_SIZE) {
        //        Serial.println("Received NTP Response");
        udp.read(packetBuffer, NTP_PACKET_SIZE);  // Read packet into the buffer
        unsigned long secsSince1900;

        // Convert four bytes starting at location 40 to a long integer
        secsSince1900 =  (unsigned long) packetBuffer[40] << 24;
        secsSince1900 |= (unsigned long) packetBuffer[41] << 16;
        secsSince1900 |= (unsigned long) packetBuffer[42] << 8;
        secsSince1900 |= (unsigned long) packetBuffer[43];

        Serial.println("Got NTP time");

        return secsSince1900 - 2208988800UL + realTimeZoneOffset * SECS_PER_HOUR;
      }
      delay(1);
    }
    Serial.println("Retrying NTP request");
    delay(10000);
  }
  Serial.println("No NTP Response");
  return 0;
}

// Assign timezone and the time sync provider
void initNTP(int _timeZoneOffset) {

  // Save incoming
  baseTimeZoneOffset = _timeZoneOffset;
  realTimeZoneOffset = _timeZoneOffset;


  // If DST is enabled, read the DST indicator from EEPROM
  if (USE_DST) {
    dstEnabled = getDSTIndicator();

    if (dstEnabled) {
      realTimeZoneOffset = baseTimeZoneOffset + 1;
    } else  {
      realTimeZoneOffset = baseTimeZoneOffset;
    }
  }
  // Set the time provider to NTP
  setSyncProvider(getNTPTime);
}

/*******************************************************************/
/***                     Support Functions                       ***/
/*******************************************************************/

// Toggle the DST enabled flag
void toggleDSTEnabled() {

  // Toggle the enabled indicator
  dstEnabled = (dstEnabled == true) ? false : true;

  // Store the enabled indicator into the EEPROM
  // so it survives a power outage
  setDSTIndicator(dstEnabled);

  // DST enabled so adjust timezone for next time sync
  // and adjust current time for immediate display
  if (dstEnabled) {
    // DST just turned on
    realTimeZoneOffset = baseTimeZoneOffset + 1;
    adjustTime(+SECS_PER_HOUR);
  } else  {
    // DST just turned off
    realTimeZoneOffset = baseTimeZoneOffset;
    adjustTime(-SECS_PER_HOUR);
  }
}

// Get the DST string
const char *getDSTStr() {
  if (dstEnabled && USE_DST) {
    return "DST";
  } else {
    return "";
  }
}

// Get the day of the week string
const char *getDWStr() {
  static char buffer[20];
  strcpy(buffer, dayShortStr(weekday()));
  return buffer;
}

// Build and return the date string
const char *getDateStr() {
  static char buffer[20];
  sprintf(buffer, "%s %d, %d", monthShortStr(month()), day(), year());
  return buffer;
}

// Build and return the time string
const char *getTimeStr() {
  int hours, minutes;
  char ampmBuffer[5];
  static char buffer[20];

  minutes = minute();

  if (HOUR_FORMAT_12) {
    hours = hourFormat12();
    if (isPM()) {
      strcpy(ampmBuffer, "PM");
    } else  {
      strcpy(ampmBuffer, "AM");
    }
    sprintf(buffer, "%d:%02d %s", hours, minutes, ampmBuffer);
  } else  {
    hours = hour();
    sprintf(buffer, "%d:%02d", hours, minutes);
  }
  return buffer;
}

// Determine day time or night time for proper icon selection
// Do comparison in 24 hour format only
// I define daytime roughly as the hours between:
//   12 hour time: 6:00AM till 8:00PM
//   24 hour time: 6 till 20
bool isDayTime() {
  int hours = hour();
  if ((hours >= 6) && (hours < 20)) {
    return true;
  } else  {
    return false;
  }
}

#endif

