/*
 *  Weather.cpp - Weather Class Implementation
 *  
 *  This class streams the weather data from the Internet,
 *  parses the JSON that is returned and extracts the pertinent
 *  weather attributes used by this program.
 *
 *  Concept, Design and Implementation by: Craig A. Lindley
 *  Last Update: 10/30/2015
 */

#include "Weather.h"

#define HTTPPORT 80
#define WEATHER_DATA_HOST "www.myweather2.com"

// Class Constructor
Weather::Weather(const char *_apiKey, const char *_locationString) {
  // Save incoming
  apiKey = _apiKey;
  locationString = _locationString;
}

// Retrieve the weather data from the weather data provider
// and parse out and save the items of interest
bool Weather::retrieveWeatherData() {

  // Mark the JSON string buffer empty
  buffer[0] = '\0';

  //  Serial.print("Connecting to: ");
  //  Serial.println(WEATHER_DATA_HOST);

  // Use WiFiClient class to create TCP connections
  if (! client.connect(WEATHER_DATA_HOST, HTTPPORT)) {
    Serial.println("No connection to weather host");
    return false;
  }

  // Typical URL: http://www.myweather2.com/developer/forecast.ashx?uac=yD9.fEaXf/&output=json&temp_unit=f&ws_unit=mph&query=80908

  // We now create a URI for the request
  String url = "/developer/forecast.ashx?uac=";
  url += apiKey;
  url += "&output=json&temp_unit=f&ws_unit=mph&query=";
  url += locationString;

  //  Serial.print("Requesting URL: ");
  //  Serial.println(url);

  // This will send the request to the server
  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
               "Host: " + WEATHER_DATA_HOST + "\r\n" +
               "Connection: close\r\n\r\n");
  delay(1000);

  // Read all the lines of the reply and store them
  while (client.available()) {
    String line = client.readStringUntil('\r');
    strcat(buffer, line.c_str());
    delay(1);
  }

  // Ignore characters before start of json string
  char *jsonString = strchr(buffer, '{');

  if ((jsonString == NULL) || (strlen(jsonString) == 0)) {
    Serial.println("Json weather string empty");
    return false;
  }

  //  Serial.print("jsonString: ");
  //  Serial.println(jsonString);

  // Create buffer for json parser
  StaticJsonBuffer<3000> jsonBuffer;

  // Attempt to parse the json weather data string
  JsonObject& root = jsonBuffer.parseObject(jsonString);

  if (! root.success()) {
    Serial.println("JSON parse failed");
    return false;
  }

  // Parse successful. Extract items of interest
  strcpy(humidity, root["weather"]["curren_weather"][0]["humidity"]);
  strcpy(pressure, root["weather"]["curren_weather"][0]["pressure"]);
  strcpy(temperature, root["weather"]["curren_weather"][0]["temp"]);
  strcpy(weatherCode, root["weather"]["curren_weather"][0]["weather_code"]);
  strcpy(weatherText, root["weather"]["curren_weather"][0]["weather_text"]);
  strcpy(windDirection, root["weather"]["curren_weather"][0]["wind"][0]["dir"]);
  strcpy(windSpeed, root["weather"]["curren_weather"][0]["wind"][0]["speed"]);
  delay(1);
  strcpy(forecastDay1Date, root["weather"]["forecast"][0]["date"]);
  strcpy(forecastDay1WeatherCode, root["weather"]["forecast"][0]["day"][0]["weather_code"]);
  strcpy(forecastDay1WeatherText, root["weather"]["forecast"][0]["day"][0]["weather_text"]);
  strcpy(forecastDay1WindDirection, root["weather"]["forecast"][0]["day"][0]["wind"][0]["dir"]);
  strcpy(forecastDay1WindSpeed, root["weather"]["forecast"][0]["day"][0]["wind"][0]["speed"]);
  strcpy(forecastDay1MaxTemperature, root["weather"]["forecast"][0]["day_max_temp"]);
  delay(1);
  strcpy(forecastNight1WeatherCode, root["weather"]["forecast"][0]["night"][0]["weather_code"]);
  strcpy(forecastNight1WeatherText, root["weather"]["forecast"][0]["night"][0]["weather_text"]);
  strcpy(forecastNight1WindDirection, root["weather"]["forecast"][0]["night"][0]["wind"][0]["dir"]);
  strcpy(forecastNight1WindSpeed, root["weather"]["forecast"][0]["night"][0]["wind"][0]["speed"]);
  strcpy(forecastNight1MinTemperature, root["weather"]["forecast"][0]["night_min_temp"]);
  delay(1);
  strcpy(forecastDay2Date, root["weather"]["forecast"][1]["date"]);
  strcpy(forecastDay2WeatherCode, root["weather"]["forecast"][1]["day"][0]["weather_code"]);
  strcpy(forecastDay2WeatherText, root["weather"]["forecast"][1]["day"][0]["weather_text"]);
  strcpy(forecastDay2WindDirection, root["weather"]["forecast"][1]["day"][0]["wind"][0]["dir"]);
  strcpy(forecastDay2WindSpeed, root["weather"]["forecast"][1]["day"][0]["wind"][0]["speed"]);
  strcpy(forecastDay2MaxTemperature, root["weather"]["forecast"][1]["day_max_temp"]);
  delay(1);
  strcpy(forecastNight2WeatherCode, root["weather"]["forecast"][1]["night"][0]["weather_code"]);
  strcpy(forecastNight2WeatherText, root["weather"]["forecast"][1]["night"][0]["weather_text"]);
  strcpy(forecastNight2WindDirection, root["weather"]["forecast"][1]["night"][0]["wind"][0]["dir"]);
  strcpy(forecastNight2WindSpeed, root["weather"]["forecast"][1]["night"][0]["wind"][0]["speed"]);
  strcpy(forecastNight2MinTemperature, root["weather"]["forecast"][1]["night_min_temp"]);
  delay(1);

  return true;
}

const char * Weather::getCurrentHumidityString() {
  strcpy(formatBuffer, "Humidity: ");
  strcat(formatBuffer, humidity);
  strcat(formatBuffer, " %");
  return formatBuffer;
}

const char * Weather::getCurrentPressureString() {
  strcpy(formatBuffer, "Pressure: ");
  strcat(formatBuffer, pressure);
  strcat(formatBuffer, " mb");
  return formatBuffer;
}

const char * Weather::getCurrentTemperatureString() {
  strcpy(formatBuffer, temperature);
  strcat(formatBuffer, " F");
  return formatBuffer;
}

const char * Weather::getCurrentWeatherCodeString() {
  return weatherCode;
}

const char * Weather::getCurrentWeatherTextString() {
  return weatherText;
}

const char * Weather::getCurrentWindDirectionString() {
  strcpy(formatBuffer, "Wind Dir: ");
  strcat(formatBuffer, windDirection);
  return formatBuffer;
}

const char * Weather::getCurrentWindSpeedString() {
  strcpy(formatBuffer, "Wind Speed: ");
  strcat(formatBuffer, windSpeed);
  strcat(formatBuffer, " mph");
  return formatBuffer;
}

const char * Weather::getForecastDay1DateString() {
  // Put the date into usual US format
  formatBuffer[0] = forecastDay1Date[5];
  formatBuffer[1] = forecastDay1Date[6];
  formatBuffer[2] = '/';
  formatBuffer[3] = forecastDay1Date[8];
  formatBuffer[4] = forecastDay1Date[9];
  formatBuffer[5] = '/';
  formatBuffer[6] = forecastDay1Date[0];
  formatBuffer[7] = forecastDay1Date[1];
  formatBuffer[8] = forecastDay1Date[2];
  formatBuffer[9] = forecastDay1Date[3];
  formatBuffer[10] = '\0';

  return formatBuffer;
}

const char * Weather::getForecastDay1WeatherCodeString() {
  return forecastDay1WeatherCode;
}

const char * Weather::getForecastDay1WeatherTextString() {
  return forecastDay1WeatherText;
}

const char * Weather::getForecastDay1WindDirectionString() {
  strcpy(formatBuffer, "Wind Dir: ");
  strcat(formatBuffer, forecastDay1WindDirection);
  return formatBuffer;
}

const char * Weather::getForecastDay1WindSpeedString() {
  strcpy(formatBuffer, "Wind Speed: ");
  strcat(formatBuffer, forecastDay1WindSpeed);
  strcat(formatBuffer, " mph");
  return formatBuffer;
}

const char * Weather::getForecastDay1MaxTemperatureString() {
  strcpy(formatBuffer, "Max: ");
  strcat(formatBuffer, forecastDay1MaxTemperature);
  strcat(formatBuffer, " F");
  return formatBuffer;
}

const char * Weather::getForecastNight1WeatherCodeString() {
  return forecastNight1WeatherCode;
}

const char * Weather::getForecastNight1WeatherTextString() {
  return forecastNight1WeatherText;
}

const char * Weather::getForecastNight1WindDirectionString() {
  strcpy(formatBuffer, "Wind Dir: ");
  strcat(formatBuffer, forecastNight1WindDirection);
  return formatBuffer;
}

const char * Weather::getForecastNight1WindSpeedString() {
  strcpy(formatBuffer, "Wind Speed: ");
  strcat(formatBuffer, forecastNight1WindSpeed);
  strcat(formatBuffer, " mph");
  return formatBuffer;
}

const char * Weather::getForecastNight1MinTemperatureString() {
  strcpy(formatBuffer, "Min: ");
  strcat(formatBuffer, forecastNight1MinTemperature);
  strcat(formatBuffer, " F");
  return formatBuffer;
}

const char * Weather::getForecastDay2DateString() {
  // Put the date into usual US format
  formatBuffer[0] = forecastDay2Date[5];
  formatBuffer[1] = forecastDay2Date[6];
  formatBuffer[2] = '/';
  formatBuffer[3] = forecastDay2Date[8];
  formatBuffer[4] = forecastDay2Date[9];
  formatBuffer[5] = '/';
  formatBuffer[6] = forecastDay2Date[0];
  formatBuffer[7] = forecastDay2Date[1];
  formatBuffer[8] = forecastDay2Date[2];
  formatBuffer[9] = forecastDay2Date[3];
  formatBuffer[10] = '\0';

  return formatBuffer;
}

const char * Weather::getForecastDay2WeatherCodeString() {
  return forecastDay2WeatherCode;
}

const char * Weather::getForecastDay2WeatherTextString() {
  return forecastDay2WeatherText;
}

const char * Weather::getForecastDay2WindDirectionString() {
  strcpy(formatBuffer, "Wind Dir: ");
  strcat(formatBuffer, forecastDay2WindDirection);
  return formatBuffer;
}

const char * Weather::getForecastDay2WindSpeedString() {
  strcpy(formatBuffer, "Wind Speed: ");
  strcat(formatBuffer, forecastDay2WindSpeed);
  strcat(formatBuffer, " mph");
  return formatBuffer;
}

const char * Weather::getForecastDay2MaxTemperatureString() {
  strcpy(formatBuffer, "Max: ");
  strcat(formatBuffer, forecastDay2MaxTemperature);
  strcat(formatBuffer, " F");
  return formatBuffer;
}

const char * Weather::getForecastNight2WeatherCodeString() {
  return forecastNight2WeatherCode;
}

const char * Weather::getForecastNight2WeatherTextString() {
  return forecastNight2WeatherText;
}

const char * Weather::getForecastNight2WindDirectionString() {
  strcpy(formatBuffer, "Wind Dir: ");
  strcat(formatBuffer, forecastNight2WindDirection);
  return formatBuffer;
}

const char * Weather::getForecastNight2WindSpeedString() {
  strcpy(formatBuffer, "Wind Speed: ");
  strcat(formatBuffer, forecastNight2WindSpeed);
  strcat(formatBuffer, " mph");
  return formatBuffer;
}

const char * Weather::getForecastNight2MinTemperatureString()  {
  strcpy(formatBuffer, "Min: ");
  strcat(formatBuffer, forecastNight2MinTemperature);
  strcat(formatBuffer, " F");
  return formatBuffer;
}


